/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.asterix.external.library.java.base;

import java.io.DataOutput;

import org.apache.asterix.dataflow.data.nontagged.serde.AStringSerializerDeserializer;
import org.apache.asterix.om.base.AMutableString;
import org.apache.asterix.om.base.AString;
import org.apache.asterix.om.types.ATypeTag;
import org.apache.asterix.om.types.BuiltinType;
import org.apache.asterix.om.types.IAType;
import org.apache.hyracks.api.exceptions.HyracksDataException;

public final class JString extends JObject<String> {

    public JString() {
        super(new AMutableString(""));
    }

    public JString(String v) {
        super(new AMutableString(v));
    }

    public void setValue(String v) {
        ((AMutableString) value).setValue(v);
    }

    public String getValue() {
        return ((AMutableString) value).getStringValue();
    }

    public String getValueGeneric() {
        return getValue();
    }

    @Override
    public void serialize(DataOutput dataOutput, boolean writeTypeTag) throws HyracksDataException {
        serializeTypeTag(writeTypeTag, dataOutput, ATypeTag.STRING);
        AStringSerializerDeserializer.INSTANCE.serialize((AString) value, dataOutput);
    }

    @Override
    public void reset() {
        ((AMutableString) value).setValue("");
    }

    @Override
    public IAType getIAType() {
        return BuiltinType.ASTRING;
    }

    @Override
    public void setValueGeneric(String o) {
        setValue(o);
    }
}
